CompactPair

Struct CompactPair 

#[repr(C)]
pub struct CompactPair { pub _address: u8, }
CompactPair is the logical concatenation of an instance of A with an instance B. Space is conserved when possible. Neither A nor B may be a final class.

In general if space conservation is not critical is preferred to use std::pair.

It’s typically clearer to have individual A and B member fields. Except if you want the space-conserving qualities of CompactPair, you’re probably better off not using this!

No guarantees are provided about the memory layout of A and B, the order of initialization or destruction of A and B, and so on. (This is approximately required to optimize space usage.) The first/second names are merely conceptual!

Fields

_address: u8