pub type SliceBudget_InterruptRequestFlag = u32;Expand description
Atomic
The atomic store and load operations and the atomic swap method is provided.
Note:
-
sizeof(Atomic
) != sizeof(bool) for some implementations of bool and/or some implementations of std::atomic. This is allowed in [atomic.types.generic]p9. -
It’s not obvious whether the 8-bit atomic functions on Windows are always inlined or not. If they are not inlined, the corresponding functions in the runtime library are not available on Windows XP. This is why we implement Atomic
with an underlying type of uint32_t.